#include "OgrePlatformQueryManager.h"
#include "OcclusionQuery.h"
#include <OgreSceneManager.h>
#include <OgreLogManager.h>
#include <OgreStringConverter.h>
#include <vector>


namespace Ogre {
//----------------------------------------------------------------------- 
PlatformQueryManager::PlatformQueryManager(PlatformHierarchyInterface *hierarchyInterface, Viewport *vp):
QueryManager(hierarchyInterface), mViewport(vp), mWasInitialised(false)
{
}
//-----------------------------------------------------------------------
bool PlatformQueryManager::ShootRay(const Ray &ray, std::vector<Mesh *> *visibleMeshes, bool isGlobalLine)
{
    // run OGRE ray shooting query
    return false;
}
//-----------------------------------------------------------------------
void PlatformQueryManager::ComputeFromPointVisibility(const Vector3 &point,
			       InfoContainer<GtpVisibility::NodeInfo> *visibleNodes,
			       InfoContainer<GtpVisibility::MeshInfo> *visibleGeometry,
				   InfoContainer<GtpVisibility::PatchInfo> *visiblePatches,
			       bool relativeVisibility)
{
	SceneManager *sm = dynamic_cast<PlatformHierarchyInterface *>
		(mHierarchyInterface)->GetSceneManager();

	// create a camera for the point query
	Camera *cam = sm->createCamera("PointQueryCam");	

	//save old camera
	Camera *savedCam = mViewport->getCamera();
	
	// --- initialise new camera
	mViewport->setCamera(cam);
	cam->setPosition(point);

	cam->setNearClipDistance(savedCam->getNearClipDistance());
	cam->setFarClipDistance(savedCam->getFarClipDistance());

	// set frustum to 45 degrees so all the scene can be captured with 6 shots
	cam->setAspectRatio(1.0);
	cam->setFOVy(Radian(Math::HALF_PI));

	int sign = -1;
	
	// ---- capture visibility from all 6 directions
	for (int dir=0; dir < 6; dir++)	
	{
		sign *= -1;
		
		// Print camera details
      	std::stringstream d;
		d << "Point query camera: " + StringConverter::toString(cam->getDerivedPosition()) + 
			" " + "O: " + StringConverter::toString(cam->getDerivedOrientation());
		LogManager::getSingleton().logMessage(d.str());

		// prevent from initialising geometry / node array again
		if (dir > 0)
		{
			mWasInitialised = true;
		}

		ComputeCameraVisibility(*cam, visibleNodes, visibleGeometry, visiblePatches, relativeVisibility);
		
		//mViewport->getTarget()->update(); for(int j=0; j<10000000; j++)	printf("wait");

		// permute directions
		Vector3 direction(0,0,0);
		direction[dir/2] = sign;

		cam->setDirection(direction);
	}
	
	// reset camera
	mViewport->setCamera(savedCam);
}
//-----------------------------------------------------------------------
void PlatformQueryManager::SetViewport(Viewport *vp)
{
	mViewport = vp;
}
} // namespace Ogre
